#! /usr/bin/env node
/*
 * Copyright (c) 2018 Includable.
 * Created by Thomas Schoffelen.
 */

global.containerEndpoint = 'https://api.sadev.io/'
global.containerVersion = 11
global.apiStability = 'stable'
global.apiVersion = '1.0'
global.verbose = false
// noinspection JSUnresolvedVariable
global.base = __dirname

const updateNotifier = require('update-notifier')
const program = require('commander')
const pkg = require('./package.json')
const commands = require('./lib/commands/index')
const plugins = require('./lib/util/Plugins')

// Replace default help screen
require('./lib/util/CommandHelp')(program, plugins)

// Check for updates
updateNotifier({ pkg }).notify()

// New module
program
  .command('create [kit] [slug]')
  .alias('start')
  .description('Create a new module')
  .action(commands.create)

// Run in sandbox
program
  .command('run')
  .description('Run module in sandbox')
  .option('-e --environment [environment]', 'Environment to run in (user,manager,main)', /^(user|manager|main)$/)
  .option('-c --container [container]', 'Specify a custom container ID', /^[a-zA-Z0-9]{8,15}$/)
  .option('-r --recipe [recipe]', 'Use a JSON recipe file to create the container from template')
  .option('-b --beta', 'Enable beta API flag')
  .option('-s --silent', 'Don\'t open in web browser')
  .option('-v --verbose', 'Verbose logging')
  .action(commands.run)

// Login
program
  .command('login')
  .description('Sign in to your account')
  .option('--token', 'Output access token')
  .action(commands.login)

// Modules commands
program
  .command('modules:list')
  .alias('modules')
  .description('List modules')
  .action(commands.modules.list)
program
  .command('modules:publish [name]')
  .alias('publish')
  .description('Publish new version of module')
  .action(commands.modules.publish)
program
  .command('modules:deploy <domain>')
  .alias('deploy')
  .description('Deploy the module on a specific domain')
  .action(commands.modules.deploy)

// Communities commands
program
  .command('communities:list')
  .alias('communities')
  .description('List communities')
  .action(commands.communities.list)

// Reset
program
  .command('reset')
  .description('Reset container')
  .action(commands.reset)

// Upgrade
program
  .command('upgrade')
  .description('Upgrade project structure')
  .action(commands.upgrade)

// Help
program.addImplicitHelpCommand = function () {}
program
  .command('help', 'View this help section')
  .action(function () {
    program.help()
  })

// Plugin commands
program
  .command('plugins:list')
  .alias('plugins')
  .description('List installed plugins')
  .action(commands.plugins.list)
program
  .command('plugins:add <plugin>')
  .alias('plugins:install <plugin>')
  .description('Install a plugin')
  .action(commands.plugins.install)
program
  .command('plugins:rm <plugin>')
  .alias('plugins:uninstall <plugin>')
  .description('Remove an installed plugin')
  .action(commands.plugins.remove)

// Download helper
program
  .command('helper:download')
  .alias('download-helper')
  .description('Download IDE helper file')
  .action(commands.downloadHelper)

// Run plugins
plugins.applyPlugins(program)

// Execute program
program
  .version('Includable CLI ' + pkg.version, '-V, --version')
  .parse(process.argv)

if (program.args.length === 0) {
  program.help()
  process.exit(1)
}
