/*
 * Copyright (c) 2018 Includable.
 * Created by Thomas Schoffelen.
 */

'use strict'

const fs = require('fs')
const inquirer = require('inquirer')
const util = require('../util')
const output = require('../output')
const Kits = require('../kits/Kits')
const Kit = require('../kits/Kit')

module.exports = function (kit, slug) {
  inquirer.registerPrompt('autocomplete', require('inquirer-autocomplete-prompt'))
  var questions = []

  if (!kit && !slug && fs.existsSync('module.json')) {
    output.warn('Are you sure you want to create a new module here? There already seems to be one in ' +
      'this directory. Instead, did you mean to type `inc run`?\n')
  } else {
    output.newline()
  }

  if (!slug || !slug.length) {
    var userName = (util.getSystemUsername() || 'user').toLowerCase()
    questions.push({
      type: 'input',
      name: 'slug',
      default: userName + '/my-module',
      message: 'What should we name this module?',
      validate: function (value) {
        if (value.match(/^([a-z0-9-]+\/)?[a-z0-9-]+$/)) {
          return true
        }

        return 'Module name should only contain lowercase letters, numbers and dashes.'
      }
    })
  }

  if (!kit || !kit.length) {
    questions.push({
      type: 'autocomplete',
      name: 'kit',
      message: 'What template would you like to use?',
      source: function (answers, input) {
        return new Promise(function (resolve) {
          Kits.list(function (list) {
            resolve(input && input.length ? list.filter(function (value) {
              return value.indexOf(input) >= 0
            }) : list)
          })
        })
      },
      default: 'basic'
    })
  }

  // Prompt questions
  inquirer.prompt(questions).then(function (answers) {
    output.newline()
    Kit.create(answers.kit || kit, answers.slug || slug)
  })
}
