/*
 * Copyright (c) 2018 Includable.
 * Created by Thomas Schoffelen.
 */

const fs = require('fs')
const path = require('path')
const mime = require('mime')
const FormData = require('form-data')
const axios = require('axios')

const pkg = require('../../package.json')

const maxDataSize = 104857600 // 100 MB

function Request (method, fields, headers, options) {
  this.method = method || ''
  this.headers = headers || {}
  this.options = options || {}

  this.data = new FormData({maxDataSize})
  if (fields) {
    for (var x in fields) {
      if (!fields.hasOwnProperty(x)) {
        continue
      }
      this.data.append(x, fields[x])
    }
  }
}

Request.prototype.attach = function (name, filename) {
  this.data.append(name, fs.createReadStream(filename), {
    filename: path.basename(filename),
    contentType: mime.getType(filename) || 'application/octet-stream'
  })

  return this
}

Request.prototype.end = function (callback) {
  axios(Object.assign({
    url: global.containerEndpoint + this.method,
    method: 'POST',
    data: this.data,
    maxContentLength: maxDataSize,
    headers: Object.assign({
      'Accept': 'application/json',
      'User-Agent': 'IncludableCLI ' + pkg.name + '/' + pkg.version,
      'X-Client': pkg.name + '/' + pkg.version,
      'X-Client-Version': pkg.version,
      'X-API-Version': global.apiVersion,
      'X-API-Stability': global.apiStability
    }, Object.assign({}, this.data.getHeaders(), this.headers || {}))
  }, this.options || {})).then(function (res) {
    if (callback) {
      callback({body: res.data, ok: true})
    }
  }, function (err) {
    if (callback) {
      callback({ok: false, error: err.message || err, body: null})
    }
  })
}

module.exports = Request
