/*
 * Copyright (c) 2018 Includable.
 * Created by Thomas Schoffelen.
 */

function humanReadableArgName (arg) {
  var nameOutput = arg.name + (arg.variadic === true ? '...' : '')

  return arg.required
    ? '<' + nameOutput + '>'
    : '[' + nameOutput + ']'
}

function pad (str, width) {
  var len = Math.max(0, width - str.length)
  return str + Array(len + 1).join(' ')
}

module.exports = function (program, plugins) {
  program.commandHelp = function () {
    if (!this.commands.length) {
      return ''
    }

    var commands = this.commands.filter(function (cmd) {
      return !cmd._noHelp
    }).map(function (cmd) {
      var args = cmd._args.map(function (arg) {
        return humanReadableArgName(arg)
      }).join(' ')

      return [
        cmd._name +
        (cmd.options.length ? ' [options]' : '') +
        (args ? ' ' + args : ''),
        cmd._description
      ]
    })

    var width = commands.reduce(function (max, command) {
      return Math.max(max, command[0].length)
    }, 0)

    var output = [
      '',
      '  Commands:',
      '',
      commands.map(function (cmd) {
        var desc = cmd[1] ? '  ' + cmd[1] : ''
        return (desc ? pad(cmd[0], width) : cmd[0]) + desc
      }).join('\n').replace(/^/gm, '    '),
      ''
    ]

    if (plugins.plugins.length) {
      output = output.concat([
        '',
        '  Installed plugins:',
        '',
        plugins.plugins.join('\n').replace(/^/gm, '    '),
        ''
      ])
    }

    return output.concat(['']).join('\n')
  }
}
